/**
 * Examples of Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

var tas = require('../../tas');

var env = {};
var a;

var tasks = function(){
	if (env.count === 2) return tas.abort('abort at 2nd');

	a = 1;

	tas(function(){
		a ++; // 2
		console.log(a);
		setTimeout(tas.next, 0);
	});

	// The tasks in tas.all() will be performed concurrently,
	// the whole task time is the longest task time.
	tas.all({
		t1: function(){
			a ++; // 3
			console.log(a);
			if (1) return tas.abort('abort');
			setTimeout(this.done, 10);
		},

		t2: function(){
			a ++; // 4
			console.log(a);
			setTimeout(this.done, 30);
		},

		t3: function(){
			a ++; // 5
			console.log(a);
			setTimeout(this.done, 20);
		},
	});

	tas(function(err, data){ // ignored
		console.log(data.length);
		console.log(a);
		tas.next();
	});
};

var exports = {
	do: function(count){

		// Save the environment variables to this module.
		// Note that we should place this operation in a function and
		// join it into Tas queue, otherwise the environment variables
		// will be changed immediately instead of waiting for its turn.
		tas(function(){
			env.count = count;
			tas.next();
		});

		// Join the tasks to Tas queue.
		// Tas will perform it in the order we arranged
		// instead of running it immediately.
		tas(tasks);

		// For require(..).do()
		return this;
	},

	get: function(){
		return a; // 2
	}
};

module.exports = exports;
