/**
 * Examples of Tas.js
 * (c) 2017 Owen Luke
 * https://github.com/tasjs/tas
 * Released under the MIT License.
 */

var tas = require('../../tas');

var env = {};
var a;

var tasks = function(){
	if (env.count === 1) return tas.abort();

	a = 1;

	tas(function(){
		a ++; // 2
		setTimeout(tas.next, 0);
	});

	// The tasks in tas.all() will be performed concurrently,
	// the whole task time is the longest task time.
	tas.all({
		t1: function(){
			a ++; // 3
			setTimeout(this.done, 10);
		},

		t2: function(){
			a ++; // 4
			setTimeout(this.done, 30);
		},

		t3: function(){
			a ++; // 5
			setTimeout(this.done, 20);
		},
	});

	tas(function(err, data){
		if (err) return tas.abort();
		// console.log(data.length);
		// console.log(a); // 5
		tas.next();
	});
};

var exports = {
	do: function(count){

		// Save the environment variables to this module.
		// Note that we should place this operation in a function and
		// join it into Tas queue, otherwise the environment variables
		// will be changed immediately instead of waiting for its turn.
		tas(function(){
			env.count = count;
			tas.next();
		});

		// Join the tasks to Tas queue.
		// Tas will perform it in the order we arranged
		// instead of running it immediately.
		tas(tasks);

		// For require(..).do()
		return this;
	},

	get: function(){
		return a; // 5
	}
};

module.exports = exports;
